#!/usr/bin/perl
#
#  Script for searching package descriptions. The result is generated as HTML.
#
#  This file is part of the SuSE help system.
#
#  Copyright (C) 2002  SuSE Linux AG, Nuernberg
#
#  Author: Cornelius Schumacher <cschum@suse.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#use strict;

use FindBin;
use lib $FindBin::Bin;

use constant DESCRPATH => "/var/adm/current_package_descr/suse/setup/descr/";

use susehelp;

sub getNext(*$);

&checkQuery;
&parseQuery;

$language = lc getQuery( "lang", "en" );
$words = getQuery( "words" );
$number = getQuery( "number", 5 );
$cgiprefix = getQuery( "cgiprefix", "cgi:" );

if ( !$words ) {
  print "No search words given.\n";
  exit;
}
@words = split /\+/, $words;

$show_url_prefix = "${cgiprefix}pacshow.cgi" . "?" . "lang=$language" .
                   "&cgiprefix=$cgiprefix" . "&package=";

# load translations
setLanguage( $language, "pac.po" );
setLanguage( $language, "susesearch.po" );

#
# now open language specific pkd file and search for words
unless (open(PKDLOCALE, DESCRPATH . $language . ".pkd")) {
  open(PKDLOCALE, DESCRPATH . "en.pkd") or die "could not even open en.pkd (as fallback)";
}

my @foundpacs;
my $current;
my $found = 0;

while (<PKDLOCALE>) {

  next unless (/^Filename:\s*(.*)-[^-]*-[^-]*\.(.*)\.rpm$/);

  $package = $1;
  $arch = $2;
  $description = getNext( PKDLOCALE, "Label:" );

  # check for matching search words
  $found = 0;
  WORD: foreach (@words) {
    if ( $arch ne "src" && ( $package =~ /$_/i || $description =~ /$_/i ) ) {
      $found = 1;
      last WORD;
    }
  }
  if ( $found ) {
    push @foundpacs, [ $package, $description ];
  }
}

# Print CGI header
print "Content-Type: text/html\n\n";

# Print HTML header
print "<html><head><title>Package Descriptions - Search Result</title></head>";
print "<body>\n";

# Print search result header
$nummatches = @foundpacs;

printSearchResultHeader( $nummatches, $number );

if ( $nummatches > 0 ) {
  print "<ul>\n";

  $count = 0;
  for $pac ( @foundpacs ) {
    ( $package, $description ) = @$pac;
    print "<li><a href=\"$show_url_prefix$package\">";
    print "$package - $description</a></li>\n";
    if ( ++$count == $number ) { last; }
  }

  print "</ul>\n";
}


# Print footer
print "</body></html>\n";

close(PKDLOCALE) or die "could not close " . $language . ".pkd";
exit 0;

#
# getNext
# return value of next entry with the given name
sub getNext(*$) {
  my $filehandle = shift;
  my $word = shift;

  my $pos = tell $filehandle;

  while (<$filehandle>) {
    if ($_ =~ /$word\s*(.*?)\s*$/i) {
      seek($filehandle, $pos, 0);
      return $1;
    }
  }
  die "could not find $word";
}
