#!/usr/bin/perl

use XML::Parser;

if ( @ARGV != 2 ) {
  print STDERR "Usage xml2meta.pl <xml-source> <output-directory>\n";
  exit 1;
}

$source = $ARGV [ 0 ];
$outputDir = $ARGV [ 1 ];

#if ( -e $outputDir ) {
#  print STDERR "Output directory '$outputDir' already exists. Exiting.\n";
#  exit 1;
#}

my $parser = new XML::Parser( ErrorContext => 2 );

$parser->setHandlers( Char => \&char_handler,
                      Default => \&default_handler,
                      Start => \&start_handler,
                      End => \&end_handler );

$parser->parsefile( $source );

sub start_handler()
{
  shift;
  my $e = shift;

  my %attribute;

  while ( @_ ) {
    my $key = shift;
    my $value = shift;
    $attribute{ $key } = $value;
  }

  my $a = $attribute{ "lang" };
  if ( defined $a ) { $lang = $a; }
  else { $lang = ""; }

  if ( $e eq "docentry" ) {
    $a = $attribute{ "weight" };
    if ( defined $a ) { $weight = $a; }
    else { $weight = 0; }

    $a = $attribute{ "type" };
    if ( defined $a ) { $type = $a; }
    else { $type = 0; }

    $templateSymbol = "";
    @templates = ();
    %template = ();
    $filename = "";
    $path = "";
    %name = ();
    %summary = ();
    %docpath = ();
    $icon = "";
    $searchMethod = "";
    $search = "";
    $searchEnabledDefault = 0;
    $indexer = "";
    $indexTestFile = "";
    $khelpcenterSpecial = "";
    $identifier = "";
    $documenttype = "";
  } elsif ( $e eq "search" ) {
    $a = $attribute{ "method" };
    if ( defined $a ) { $searchMethod = $a; }

    $a = $attribute{ "enabledDefault" };
    if ( $a eq "true") { $searchEnabledDefault = 1; }
  } elsif ( $e eq "khelpcenterspecial" ) {
    $khelpcenterSpecial = $attribute{ "type" };
  } elsif ( $e eq "template" ) {
    $templateSymbol = $attribute{ "symbol" };
  }

  $text = "";
}

sub end_handler()
{
  shift;
  my $e = shift;

  if ( $e eq "value" ) {
    push @templates, $text;
  } elsif ( $e eq "filename" ) {
    $filename = $text;
  } elsif ( $e eq "path" ) {
    $path = $text;
  } elsif ( $e eq "name" ) {
    $name{ $lang } = $text;
  } elsif ( $e eq "summary" ) {
    $summary{ $lang } = $text;
  } elsif ( $e eq "docpath" ) {
    $docpath{ $lang } = $text;
  } elsif ( $e eq "icon" ) {
    $icon = $text;
  } elsif ( $e eq "identifier" ) {
    $identifier = $text;
  } elsif ( $e eq "url" ) {
    $search = $text;
  } elsif ( $e eq "indexer" ) {
    $indexer = $text;
  } elsif ( $e eq "indextestfile" ) {
    $indexTestFile = $text;
  } elsif ( $e eq "documenttype" ) {
    $documenttype = $text;
  } elsif ( $e eq "docentry" ) {
    if ( $type ne "directory" ) {
#      print "PATH: $path ENTRY: $filename  WEIGHT: $weight\n";

      if ( $templateSymbol ) {
        foreach $v ( @templates ) {
          $templateValue = $v;
          createDesktopFile( substituteTemplate( $filename ) );
        }
      } else {
        createDesktopFile( $filename );
      }
    }
  }
}

sub createDesktopFile($)
{
  my $out = shift;

  my $dir = "$outputDir$path";
  if ( !-e $dir ) {
    system ( "mkdir -p $dir" );
  }

  my $file = "$dir/$out";
  if ( !open( OUT, ">:utf8", "$file" ) ) {
    print STDERR "Can't open '$file' for writing.\n";
  } else {
    print OUT "[Desktop Entry]\n";
    if ( $type eq "category" ) { print OUT "Type=category\n"; }
    foreach $l ( sort keys %name ) {
      printLangTag( "Name", $l, $name{ $l } );
    }
    foreach $l ( sort keys %summary ) {
      printLangTag( "Comment", $l, $summary{ $l } );
    }
    foreach $l ( sort keys %docpath ) {
      printLangTag( "DocPath", $l, $docpath{ $l } );
    }
    printTag( "Icon", $icon );
    printTag( "X-DOC-Identifier", $identifier );
    printTag( "X-DOC-SearchMethod", $searchMethod );
    printTag( "X-DOC-Search", $search );
    printTag( "X-DOC-Indexer", $indexer );
    printTag( "X-DOC-IndexTestFile", $indexTestFile );
    if ( $searchEnabledDefault ) {
      print OUT "X-DOC-SearchEnabledDefault=true\n";
    }
    if ( $weight != 0 ) {
      print OUT "X-DOC-Weight=$weight\n";
    }
    printTag( "X-DOC-DocumentType", $documenttype );
    printTag( "X-KDE-KHelpcenter-Special", $khelpcenterSpecial );

    close OUT;
  }
}

sub substituteTemplate($)
{
  my $in = shift;
  
  if ( !$templateSymbol ) { return $in; }

  $in =~ s/$templateSymbol/$templateValue/g;

  return $in;
}

sub printLangTag($$$)
{
  my $key = shift;
  my $l = shift;
  my $v = shift;
  
  print OUT "$key";
  if ( $l ) { print OUT "[$l]"; }
  print OUT "=", substituteTemplate( $v ), "\n";
}  

sub printTag($$)
{
  my $key = shift;
  my $value = shift;
  
  if ( $value ) { print OUT "$key=", substituteTemplate( $value ), "\n"; }
}

sub char_handler()
{
  shift;
  my $data = shift;

  $text .= $data;
}

sub default_handler()
{

}
